/**
 * Information defined for printer models
 * 
 * @author Brother Industries, Ltd.
 * @version 2.2
 */

package com.brother.ptouch.sdk.printdemo.printprocess;

import java.util.Locale;
import com.brother.ptouch.sdk.printdemo.common.Common;

public class PrinterModelInfo {

	   /**
     * Arrays of paper sizes.
     * Element 0 is default.
     * The PocketJet variant for Locale.US is the same as non-US
     * except the order changes to set a different default.
     */
    private static final String[] PS_A6_ONLY       = { "A6" };
    private static final String[] PS_A7_ONLY       = { "A7" };
    private static final String[] PS_CUSTOM_ONLY   = { "CUSTOM" };
    private static final String[] PS_PJ            = { "A4", "LETTER", "LEGAL", "A5","A5_LANDSCAPE", "CUSTOM" };
    private static final String[] PS_PJ_US         = { "LETTER", "LEGAL", "A4", "A5", "A5_LANDSCAPE","CUSTOM" };
    private static final String[] PS_PT            = { "W3_5","W6","W9","W12","W18","W24","HS_W6","HS_W9","HS_W12","HS_W18","HS_W24" };
    private static final String[] PS_QL            = { "W17H54","W17H87","W23H23","W29H42","W29H90","W38H90","W39H48","W52H29","W62H29","W62H100","W60H86","W12","W29","W38","W50","W54","W62" };

    /**
     * Arrays of printer ports.
     * Element 0 is default.
     */
    private static final String[] PORTS_USB        = {                               Common.USB };
    private static final String[] PORTS_BT_USB     = {             Common.BLUETOOTH, Common.USB };
    private static final String[] PORTS_NET_USB    = { Common.NET,                   Common.USB };
    private static final String[] PORTS_NET_BT_USB = { Common.NET, Common.BLUETOOTH, Common.USB };

    // Port order is different for TD4 printers. 
    private static final String[] PORTS_USB_BT     = {             Common.USB, Common.BLUETOOTH };
    private static final String[] PORTS_NET_USB_BT = { Common.NET, Common.USB, Common.BLUETOOTH };

   
    
    /**
     * All printer models.
     * Each model holds a list of supported communication ports and paper sizes.
     */
    public enum Model {
		MW_140BT(PORTS_BT_USB, PS_A7_ONLY), 
		MW_145BT(MW_140BT), 
		MW_145MFi(MW_140BT), 
		MW_260(PORTS_BT_USB, PS_A6_ONLY), 
		MW_260MFi(MW_260), 
		PJ_520(PORTS_USB, PS_PJ, PS_PJ_US), 
		PJ_522(PJ_520), 
		PJ_523(PJ_520), 
		PJ_622(PJ_520), 
		PJ_623(PJ_520), 
		PJ_722(PJ_520), 
		PJ_723(PJ_520),
		PJ_560(PORTS_BT_USB, PS_PJ, PS_PJ_US), 
		PJ_562(PJ_560), 
		PJ_563(PJ_560), 
		PJ_662(PJ_560), 
		PJ_663(PJ_560), 
		PJ_762(PJ_560), 
		PJ_763(PJ_560), 
		PJ_763MFi(PJ_560),
		PJ_773(PORTS_NET_USB, PS_PJ, PS_PJ_US),
		RJ_4030(PORTS_BT_USB, PS_CUSTOM_ONLY), 
		RJ_4040(PORTS_NET_USB, PS_CUSTOM_ONLY), 
		RJ_3050(PORTS_NET_BT_USB, PS_CUSTOM_ONLY), 
		RJ_3150(RJ_3050), 
		TD_2020(PORTS_USB, PS_CUSTOM_ONLY), 
		TD_2120N(PORTS_NET_BT_USB, PS_CUSTOM_ONLY), 
		TD_2130N(TD_2120N), 
		TD_4000(PORTS_USB_BT, PS_CUSTOM_ONLY), 
		TD_4100N(PORTS_NET_USB_BT, PS_CUSTOM_ONLY), 
		QL_710W(PORTS_NET_USB, PS_QL), 
		QL_720NW(QL_710W), 
		QL_580N(QL_710W), 
		PT_E550W(PORTS_NET_USB, PS_PT), 
		PT_P750W(PT_E550W);
   
        private final String[] mPorts;
        private final String[] mPaperSizes;
        private final String[] mPaperSizesUS;

        private Model(String[] ports, String[] paperSizes, String[] paperSizesUS) {
            mPorts = ports;
            mPaperSizes = paperSizes;
            mPaperSizesUS = paperSizesUS;
        }

        private Model(String[] ports, String[] paperSizes) {
            this(ports, paperSizes, null);
        }

        private Model(Model alias) {
            this(alias.mPorts, alias.mPaperSizes, alias.mPaperSizesUS);
        }

        public String[] getPorts() {
            return mPorts;
        }

        public String[] getPaperSizes() {
            return (Locale.getDefault().equals(Locale.US) && (mPaperSizesUS != null))
                    ? mPaperSizesUS : mPaperSizes;
        }

        public String getDefaultPaperSize() {
            String[] paperSizes = getPaperSizes();
            String paperSize = (paperSizes != null) ? paperSizes[0] : "";
            return (paperSize != null) ? paperSize : "";
        }
    } // enum Model

    /**
     * String array containing the names of all printer models.
     */
    private static String[] model = null;

    /**
     * Get an array of the names of all printer models.
     * 
     * @return array of names of all printer models
     */
    public static String[] getModelNames() {
        if (model != null) {
            return model;
        }
        // Lazy initialization
        Model[] models = Model.values();
        int count = models.length;
        String[] m = new String[count];
        for (int i = 0; i < count; ++i)
        {
            m[i] = models[i].name();
        }
        model = m;
        return model;
    }

	
    /**
     * Get the port or paper size information for a specific model.
     * 
     * @param model printer model name
     * @param value type of information, either Common.SETTINGS_PORT or Common.SETTINGS_PAPERSIZE
     * @return the requested information, if input parameters are valid, otherwise return null
     */
    public static String[] getPortOrPaperSizeInfo(String model, String value)
    {
        String name = model.trim();
        Model m = null;
        String[] result = null;

        try {
            m = Model.valueOf(name);
            if      (value.equalsIgnoreCase(Common.SETTINGS_PORT))      { result = m.getPorts(); }
            else if (value.equalsIgnoreCase(Common.SETTINGS_PAPERSIZE)) { result = m.getPaperSizes(); }
        } catch (IllegalArgumentException e) {}
        return result;
    }
}	
	